/*
Flicky
	by Lach
*/

local SKIN = "flicky"

freeslot(
"s_flikded1",
"s_flikded2",
"s_flikpeck",
"s_flikdive",
"sfx_flpeck",
"sfx_fldive",
"sfx_fldead",
"spr2_dive")

states[S_FLIKDED1] = {SPR_PLAY, SPR2_DEAD|FF_ANIMATE, 10, A_None, 0, 5, S_FLIKDED2}
states[S_FLIKDED2] = {SPR_PLAY, SPR2_CNT1, 3, A_None, 0, 0, S_FLIKDED2}

states[S_FLIKPECK] = {SPR_PLAY, SPR2_MLEE|FF_ANIMATE, 10, A_None, 0, 2, S_PLAY_STND}

states[S_FLIKDIVE] = {SPR_PLAY, SPR2_DIVE, -1}

sfxinfo[sfx_flpeck].caption = "Peck"
sfxinfo[sfx_fldive].caption = "Dive"
sfxinfo[sfx_fldead].caption = "Cold-hearted bird murder"

addHook("ThinkFrame", do
	for player in players.iterate
		local mo = player.mo
		if not mo
		or mo.skin ~= SKIN
		or player.powers[pw_carry] == CR_NIGHTSMODE
			continue
		end
		
		// Death and Game Over
		if (player.playerstate == PST_DEAD)
			if (mo.state == S_PLAY_DEAD)
				mo.flags = $ & ~MF_NOCLIPHEIGHT
				mo.momz = P_MobjFlip(mo)*3*mo.scale
				S_StartSound(mo, sfx_fldead)
				if player.lives == 0
					S_StopMusic(player)
				end
				mo.state = S_FLIKDED1
			end
			if player.deadtimer == 2*TICRATE and player.lives == 0
				S_ChangeMusic(666, 0, player)
			end
			continue
		end
		
		// Dive
		if mo.state == S_FLIKDIVE
			//If Flicky is going UP, then that probably means they shouldn't be in their dive state
			if P_MobjFlip(mo)*mo.momz > 0
				player.pflags = $ & ~PF_THOKKED | P_GetJumpFlags(player)
				mo.state = S_PLAY_JUMP
			end
		else
			if not player.fly1 and P_MobjFlip(mo)*mo.momz < 0 and mo.state != S_PLAY_FLY_TIRED and not P_PlayerInPain(player)
				P_SetObjectMomZ(mo, -P_MobjFlip(mo)*mo.momz/10, true)
			end
		end
		
		// No landing lag from diving
		if mo.state == S_PLAY_MELEE_LANDING
			mo.state = states[mo.state].nextstate
		end
		
		if mo.state == S_PLAY_ROLL and not player.homing
			mo.state = S_PLAY_FALL
			player.pflags = $ | P_GetJumpFlags(player)
		end
		
		// Handles Putting Flicky in flying states
		if not (player.pflags & PF_JUMPED)
		and not (player.pflags & PF_THOKKED)
		and player.panim != PA_ABILITY2
		and not P_IsObjectOnGround(mo)
		and not player.powers[pw_carry]
		and not player.powers[pw_tailsfly]
		and not P_PlayerInPain(player)
			player.pflags = $ | P_GetJumpFlags(player)
			mo.state = S_PLAY_JUMP
		end
		
		// Flicky shoots slightly faster than Sonic
		if (player.weapondelay == 2)
			player.weapondelay = 0
		end
		
		// Can only fly for 4 seconds
		if player.powers[pw_tailsfly] == 8*TICRATE
			player.powers[pw_tailsfly] = 4*TICRATE
		end
		
		// By default, flying halves player speed, which is bad for Flicky considering their scale; this (crudely) fixes that problem
		if player.powers[pw_tailsfly]
			if not player.flicky_wasflying
				player.normalspeed = skins[mo.skin].normalspeed * 2
				player.flicky_wasflying = true
			end
		else
			if player.flicky_wasflying
				player.normalspeed = skins[mo.skin].normalspeed
				player.fly1 = 0
				player.flicky_wasflying = false
			end
		end
		
		// Peck sound
		if mo.state == S_FLIKPECK and mo.tics == 6 and P_IsObjectOnGround(mo)
			S_StartSound(mo, sfx_flpeck)
		end
	end
end)

// Spin abilities - divebomb and peck
addHook("SpinSpecial", function(player)
	local mo = player.mo
	
	if not mo
	or mo.skin ~= SKIN
	or mo.state == S_FLIKDIVE
	or mo.state == S_FLIKPECK
	or player.pflags & PF_USEDOWN
	or player.powers[pw_carry]
	or (player.pflags & PF_JUMPED and player.powers[pw_shield] & SH_NOSTACK or player.powers[pw_tailsfly]) and player.cmd.buttons & BT_JUMP
		return
	end
	
	if P_IsObjectOnGround(mo)
	and not mo.momz
		mo.state = S_FLIKPECK
		player.panim = PA_ABILITY2
		player.powers[pw_nocontrol] = mo.tics
		return true
	end
	
	if player.pflags & (PF_JUMPED|PF_THOKKED)
		local divespeed = 27*mo.scale
		if player.powers[pw_super]
			divespeed = $ * 3
		elseif player.powers[pw_sneakers]
			divespeed = $ * 2
		end
		if mo.eflags & MFE_UNDERWATER
			divespeed = 2 * $ / 3
		end
		
		P_InstaThrust(mo, mo.angle, divespeed)
		P_SetObjectMomZ(mo, FixedMul(divespeed >> 1, -sin(ANGLE_45)), false)
		S_StartSound(mo, sfx_fldive)
		mo.state = S_FLIKDIVE
		player.panim = PA_ABILITY2
		player.powers[pw_tailsfly] = 0
		player.pflags = $ & ~PF_NOJUMPDAMAGE | PF_JUMPED | PF_THOKKED
		player.drawangle = mo.angle
		return
	end
end)

// Bounce off enemies/monitors
addHook("MobjDamage", function(item, mo)
	if mo and mo.valid and item.valid
	and mo.skin == SKIN
	and mo.state == S_FLIKDIVE
	and item.flags & MF_SHOOTABLE
		mo.momz = -$
	end
end)

addHook("MobjDeath", function(item, mo)
	if mo and mo.valid and item.valid
	and mo.skin == SKIN
	and mo.state == S_FLIKDIVE
	and item.flags & MF_MONITOR
		mo.momz = -$
	end
end)